@@ -149,7 +149,7 @@ sub dispatch_to_swagger {
   for my $p (@{$op_info->{spec}{parameters} || []}) {
     my $name  = $p->{name};
     my $value = $data->{params}{$name} // $p->{default};
-    my @e     = $self->_validate_value($p, $name => $value);
+    my @e     = $self->_validate_input_value($p, $name => $value);
     $input->{$name} = $value unless @e;
     push @errors, @e;
   }
@@ -166,11 +166,7 @@ sub dispatch_to_swagger {
       my $delay  = shift;
       my $data   = shift;
       my $status = shift || 200;
-      my $format = $op_info->{spec}{responses}{$status} || $op_info->{spec}{responses}{default} || undef;
-      my @errors
-        = !$format ? $self->_validator->validate($data, {})
-        : $format->{schema} ? $self->_validator->validate($data, $format->{schema})
-        :                     ();
+      my @errors = $self->_validate_response($c, $data, $op_info->{spec}, $status);
 
       return $c->$reply($data, $status) unless @errors;
       warn "[Swagger2] Invalid response: @errors\n" if DEBUG;
@@ -414,11 +410,7 @@ sub _generate_request_handler {
         my $delay  = shift;
         my $data   = shift;
         my $status = shift || 200;
-        my $format = $op_spec->{responses}{$status} || $op_spec->{responses}{default} || undef;
-        my @errors
-          = !$format ? $self->_validator->validate($data, {})
-          : $format->{schema} ? $self->_validator->validate($data, $format->{schema})
-          :                     ();
+        my @errors = $self->_validate_response($c, $data, $op_spec, $status);
 
         return $c->render_swagger({}, $data, $status) unless @errors;
         warn "[Swagger2] Invalid response: @errors\n" if DEBUG;
@@ -491,7 +483,7 @@ sub _validate_input {
       }
     }
 
-    my @e = $self->_validate_value($p, $name => $value);
+    my @e = $self->_validate_input_value($p, $name => $value);
     $input{$name} = $value unless @e;
     push @errors, @e;
   }
@@ -499,7 +491,7 @@ sub _validate_input {
   return {errors => \@errors}, \%input;
 }
 
-sub _validate_value {
+sub _validate_input_value {
   my ($self, $p, $name, $value) = @_;
   my $type = $p->{type} || 'object';
   my @e;
@@ -511,20 +503,49 @@ sub _validate_value {
 
   if ($in eq 'body') {
     warn "[Swagger2] Validate $in $name\n" if DEBUG;
-    return $self->_validator->validate({$name => $value}, $schema);
+    return $self->_validator->validate_input({$name => $value}, $schema);
   }
   elsif (defined $value) {
     warn "[Swagger2] Validate $in $name=$value\n" if DEBUG;
-    return $self->_validator->validate({$name => $value}, $schema);
+    return $self->_validator->validate_input({$name => $value}, $schema);
   }
   else {
     warn "[Swagger2] Validate $in $name=undef\n" if DEBUG;
-    return $self->_validator->validate({$name => $value}, $schema);
+    return $self->_validator->validate_input({$name => $value}, $schema);
   }
 
   return;
 }
 
+sub _validate_response {
+  my ($self, $c, $data, $op_spec, $status) = @_;
+  my @errors;
+
+  if (my $res = $op_spec->{responses}{$status} || $op_spec->{responses}{default}) {
+    my $headers = $c->res->headers->to_hash(1);
+
+    for my $n (keys %{$res->{headers} || {}}) {
+      my $p = $res->{headers}{$n};
+
+      # jhthorsen: I think that the only way to make a header required,
+      # is by defining "array" and "minItems" >= 1.
+      if ($p->{type} eq 'array') {
+        push @errors, $self->_validator->validate($headers->{$n}, $p);
+      }
+      elsif ($headers->{$n}) {
+        push @errors, $self->_validator->validate($headers->{$n}[0], $p);
+      }
+    }
+
+    push @errors, $self->_validator->validate($data, $res->{schema}) if $res->{schema};
+  }
+  else {
+    push @errors, $self->_validator->validate($data, {});
+  }
+
+  return @errors;
+}
+
 sub _coerce_by_collection_format {
   my ($data, $schema) = @_;
   my $re = $Swagger2::SchemaValidator::COLLECTION_RE{$schema->{collectionFormat}} || '';
@@ -556,6 +577,7 @@ sub _error {
 sub _find_controller {
   my ($c, $moniker) = @_;
   my $controller = $moniker;
+  my $e;
 
   return $controller if $controller =~ /::/ and !defined load_class $controller;
   $controller =~ s!s$!!;    # plural to singular, "::Pets" to "::Pet"
@@ -563,9 +585,10 @@ sub _find_controller {
   for my $ns (@{$c->app->routes->namespaces}) {
     my $class = "${ns}::$controller";
     return $class unless defined load_class $class;
+    $e ||= $@;              # want the most specific class names
   }
 
-  $c->app->log->error(qq(Could not find controller class for "$moniker": $@));
+  $c->app->log->error(qq(Could not find controller class for "$moniker": $e));
   return;
 }
 
